
/*
*  file:   initaudio.c 
*
* initializes  audio I/O via ALSA SYSTEM
* for transfer of audio to short int  buffer
*
* PA0MBO  - M.Bos  
* Date Feb 21st 2009
*
* 
*
*/

/*************************************************************************
*
*                           PA0MBO
*
*    COPYRIGHT (C)  2009  M.Bos 
*
*    This file is part of the distribution package RXAMADRM
*
*    This package is free software and you can redistribute is
*    and/or modify it under the terms of the GNU General Public License
*
*    More details can be found in the accompanying file COPYING
*************************************************************************/

#define CHUNK 4096

#include <stdio.h>
#include <string.h>
#include <alsa/asoundlib.h>
#include <math.h>
#include <pthread.h>
#include <semaphore.h>
#include "drmdefs.h"

/* static char *device = "hw:2,0"; playback devices */ 
static char *snddevices[4] = {"hw:0,0", "hw:1,0", "hw:2,0", "hw:3:0"} ;
char *device;
static snd_pcm_format_t format = SND_PCM_FORMAT_S16_LE;	/* sample format */
static unsigned int rate = 48000;	/* stream rate */
static unsigned int nchannels = 2;	/* count of channels */
static int resample = 0;	/* disable alsa-lib resampling */
static snd_pcm_sframes_t buffer_size;
static snd_pcm_sframes_t period_size;
static snd_output_t *output = NULL;
short int tmpbuf[100000];
int bufsize = 50000;
static int startptr;
static int fillptr;
int emptyptr;
volatile int bufcnt;
static int verbose = 0;
sem_t mutex ;

static int
set_hwparams(snd_pcm_t * handle, snd_pcm_hw_params_t * params,
	     snd_pcm_access_t access)
{
  unsigned int rrate;

  snd_pcm_uframes_t size;
  int err,  dir;


  /* choose all parameters */
  err = snd_pcm_hw_params_any(handle, params);
  if (err < 0)
    {
      printf
	("Broken configuration for playback: no configurations available: %s\n",
	 snd_strerror(err));
      return err;
    }

  /* set hardware resampling  */
  err = snd_pcm_hw_params_set_rate_resample(handle, params, resample);
  if (err < 0)
    {
      printf("Resampling setup failed for record: %s\n", snd_strerror(err));
      return err;
    }

  /* set the interleaved read/write format */
  err = snd_pcm_hw_params_set_access(handle, params, access);
  if (err < 0)
    {
      printf("Access type not available for record: %s\n", snd_strerror(err));
      return err;
    }

  /* set the sample format */
  err = snd_pcm_hw_params_set_format(handle, params, format);
  if (err < 0)
    {
      printf("Sample format not available for record: %s\n",
	     snd_strerror(err));
      return err;
    }

  /* set the count of channels */
  err = snd_pcm_hw_params_set_channels(handle, params, nchannels);
  if (err < 0)
    {
      printf("Channels count (%i) not available for record: %s\n", nchannels,
	     snd_strerror(err));
      return err;
    }

  /* set the stream rate */
  rrate = rate;
  err = snd_pcm_hw_params_set_rate_near(handle, params, &rrate, 0);
  if (err < 0)
    {
      printf("Rate %iHz not available for record: %s\n", rate,
	     snd_strerror(err));
      return err;
    }
  if (rrate != rate)
    {
      printf("Rate doesn't match (requested %iHz, get %iHz)\n", rate, err);
      return -EINVAL;
    }
  size = 4 * CHUNK;
  err = snd_pcm_hw_params_set_buffer_size_near(handle, params, &size);
  if (err < 0)
    {
      printf("Unable to set buffer size %li for set near\n", size);
      exit(1);
    }
  err = snd_pcm_hw_params_get_buffer_size(params, &size);
  buffer_size = size;
  /* printf("buf_size set near %i got %i err %i \n", 4 * CHUNK, (int) size, err); */

  /* set the period size */
  size = CHUNK;
  err = snd_pcm_hw_params_set_period_size_near(handle, params, &size, &dir);
  if (err < 0)
    {
      printf("Unable to set period size  %i for playback: %s\n", (int) size,
	     snd_strerror(err));
      return err;
    }
  err = snd_pcm_hw_params_get_period_size(params, &size, &dir);
  if (err < 0)
    {
      printf("Unable to get period size for playback: %s\n",
	     snd_strerror(err));
      return err;
    }
  period_size = size;
  /* printf("period size from get %i\n", (int) size); */

  /* write the parameters to device */
  err = snd_pcm_hw_params(handle, params);
  if (err < 0)
    {
      printf("Unable to set hw params for playback: %s\n", snd_strerror(err));
      return err;
    }
  return 0;
}
static int set_swparams(snd_pcm_t * handle, snd_pcm_sw_params_t * swparams)
{
  int err;


  /* get the current swparams */
  err = snd_pcm_sw_params_current(handle, swparams);
  if (err < 0)
    {
      printf("Unable to determine current swparams for playback: %s\n",
	     snd_strerror(err));
      return err;
    }

  /* allow the transfer when at least period_size samples can be processed */
  /* or disable this mechanism when period event is enabled (aka interrupt like style processing) */
  err = snd_pcm_sw_params_set_avail_min(handle, swparams, buffer_size);
  if (err < 0)
    {
      printf("Unable to set avail min for playback: %s\n", snd_strerror(err));
      return err;
    }

  /* write the parameters to the playback device */
  err = snd_pcm_sw_params(handle, swparams);
  if (err < 0)
    {
      printf("Unable to set sw params for playback: %s\n", snd_strerror(err));
      return err;
    }
  return 0;
}



/*
* Underrun and suspend recovery
*/
static int xrun_recovery(snd_pcm_t * handle, int err)
{
  if (verbose)
    printf("stream recovery\n");
  if (err == -EPIPE)
    {				/* under-run */
      err = snd_pcm_prepare(handle);
      printf("recovered from xrun\n");
      if (err < 0)
	printf("Can't recovery from underrun, prepare failed: %s\n",
	       snd_strerror(err));
      return 0;
    }
  else if (err == -ESTRPIPE)
    {
      while ((err = snd_pcm_resume(handle)) == -EAGAIN)
	sleep(1);		/* wait until the suspend flag is released */
      if (err < 0)
	{
	  err = snd_pcm_prepare(handle);
	  if (err < 0)
	    printf("Can't recovery from suspend, prepare failed: %s\n",
		   snd_strerror(err));
	}
      return 0;
    }
  return err;
}


/*
* Transfer method - asynchronous notification
*/
static void async_callback(snd_async_handler_t * ahandler)
{
  snd_pcm_t *handle = snd_async_handler_get_pcm(ahandler);

  /* signed short *samples = snd_async_handler_get_callback_private(ahandler); */
  snd_pcm_sframes_t avail;
  int err, err2;


  /* printf("in async_call back\n"); */
  avail = snd_pcm_avail_update(handle);
  if (avail == -EPIPE)
    {

      /* EPIPE means overrun */
      printf("overrun occurred\n");
      xrun_recovery(handle, avail);
      snd_pcm_start(handle);
      return;
    }
  else if (avail < 0)
    {
      printf("error avail update\n");
      return;
    }
  err = 0;
  err2=0;

  /* printf("in async_callback avail = %d\n", (int) avail);  */

      /* printf("ready to read samples avail= %i\n", avail); */
      sem_wait(&mutex);
      if ((BUFSIZE - fillptr) > avail)    /* enough psace for complete CHUNK */
      {
           err = snd_pcm_readi(handle,tmpbuf+2*fillptr , avail);
	   fillptr += avail;
      }
      else
      {
           err = snd_pcm_readi(handle,tmpbuf+2*fillptr , BUFSIZE - fillptr);   /* to end of buffer */
           err2 = snd_pcm_readi(handle,tmpbuf, avail - BUFSIZE + fillptr);   /* remainder */
	   fillptr += avail - BUFSIZE  ;
      }

      /* printf("read %d samples \n",err); */
      if ((err < 0) || (err2 <0))
	{
	  printf("Read error: %s\n", snd_strerror(err));
	  exit(EXIT_FAILURE);
	}
        bufcnt += avail;
	sem_post(&mutex);

}
static int init_async(snd_pcm_t * handle, signed short *samples)
{
  signed short data[20000];

  snd_async_handler_t *ahandler;
  int err;

  err = snd_async_add_pcm_handler(&ahandler, handle, async_callback, &data);
  if (err < 0)
    {
      printf("Unable to register async handler\n");
      exit(EXIT_FAILURE);
    }

  /* printf("na registering async_handler\n"); */
  if (snd_pcm_state(handle) == SND_PCM_STATE_PREPARED)
    {

      /* printf(" checking state\n"); */
      err = snd_pcm_start(handle);
      if (err < 0)
	{
	  printf("Start error: %s\n", snd_strerror(err));
	  exit(EXIT_FAILURE);
	}
    }

  /* printf("end of async loop\n"); */
  return (err);
}
void init_audio(void)
{
  snd_pcm_t *handle;
  int err, morehelp, nrdev;

  snd_pcm_hw_params_t *hwparams;
  snd_pcm_sw_params_t *swparams;
  signed short *samples;
  FILE *fpini;
  char regel[80];
  char *delims="= " ;
  char *ptxt;

  /* init buffer pointer for transfers */
  startptr = 0;
  fillptr = 0;
  emptyptr = 0;
  bufcnt = 0;
  nrdev=0 ;
  /* set the sound_device  */

  fpini = fopen("rxamadrm.ini", "r");
  if (fpini == NULL)
  {
	  printf("Cannot open rxamadrm.ini \n");
	  exit(1);
  }

  while (fgets(regel, 80, fpini) != NULL)
  {
	  ptxt =strtok(regel, delims);
	  if (strcmp(ptxt,"devin")==0)
	  {
		  ptxt=strtok(NULL, delims);
		  sscanf(ptxt,"%d",&nrdev);
	  }
  }
  
  device = snddevices[nrdev] ;   /* if not found =0 */
  /* init the mutex for yje async transfers */

   sem_init(&mutex,1,1 );


  /* printf("Start startptr %d fillptr %d emptryptr %d\n", startptr, fillptr, emptyptr); */
  snd_pcm_hw_params_alloca(&hwparams);
  snd_pcm_sw_params_alloca(&swparams);
  morehelp = 0;
  rate = 48000;
  format = SND_PCM_FORMAT_S16;
  /* printf("Device is %s\n", device);
  printf("Stream parameters are %iHz, %s, %i channels\n", rate,
	 snd_pcm_format_name(format), nchannels); */
  if ((err = snd_pcm_open(&handle, device, SND_PCM_STREAM_CAPTURE, 0)) < 0)
    {
      printf("Capture open error: %s\n", snd_strerror(err));
      return;
    }
  if ((err =
       set_hwparams(handle, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
    {
      printf("Setting of hwparams failed: %s\n", snd_strerror(err));
      exit(EXIT_FAILURE);
    }
  if ((err = set_swparams(handle, swparams)) < 0)
    {
      printf("Setting of swparams failed: %s\n", snd_strerror(err));
      exit(EXIT_FAILURE);
    }
  if (verbose > 0)
    snd_pcm_dump(handle, output);
  samples =
    malloc((period_size * nchannels * snd_pcm_format_physical_width(format)) /
	   8);
  if (samples == NULL)
    {
      printf("Not enough memory\n");
      exit(EXIT_FAILURE);
    }
  err = snd_pcm_prepare(handle);
  if (err < 0)

    {
      printf("cannot prepare handle \n");
      exit(1);
    }
  init_async(handle, samples);

  /* printf("na call init_async in main\n"); */
}
