/*
*  file:   stereoreclsb.c 
*
* record signal from audio system ALSA
*
* PA0MBO  - M.Bos  
* Date Jan 1st 2013 
*
*/

/*************************************************************************
*
*                           PA0MBO
*
*    COPYRIGHT (C)  2013  M.Bos 
*
*    This file is part of the distribution package RXAMADRM
*
*    This package is free software and you can redistribute is
*    and/or modify it under the terms of the GNU General Public License
*
*    More details can be found in the accompanying file COPYING
*************************************************************************/


#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <stdio.h>
#include <math.h>
#include <alsa/asoundlib.h>
#include <semaphore.h>
#include <pthread.h>
#include <sys/types.h>
#include <malloc.h>
#include "drmdefs.h"

#define PI  (4*atan(1.000))

extern float acq_signal[2*BUFSIZE];
extern int emptyptr;
extern short int tmpbuf[100000];
extern volatile int bufcnt;
extern int bufsize;
extern int input_samples_buffer_request;
extern int inpbufwidx;
extern sem_t mutex ;
extern float HilbIfilt[LP_LENGTH];
extern float HilbQfilt[LP_LENGTH];
extern float ZifiltI[LP_LENGTH];
extern float ZifiltQ[LP_LENGTH];

void filter(float *, float *, float *, float *, int, int);

int stereorec(int nr_samps, /*@out@ */ long *bufaucnt)
{
  static int cnt = 0;
  float tempI[BUFSIZE], tempQ[BUFSIZE];
  float sI[BUFSIZE], sQ[BUFSIZE];
  int flag ;
  int i;

  if (cnt < 2048)
    bufaucnt[cnt++] = bufcnt;

  /* active waiting */


  do
  {
       sem_wait( &mutex);
       flag = 	(bufcnt < nr_samps) ; 
       sem_post(&mutex); 
       sleep(1);
  } while (flag) ;

  /* get the audiosamples int short int format in the tmpbuf to
   * the float buffer temp 
   */

  sem_wait( &mutex);
  for (i=0 ; i < nr_samps ; i++ )
  {
      tempQ[i] = (float) tmpbuf[2*emptyptr] ;    /* voor usb omgewisseld */
      tempI[i] = (float) tmpbuf[2*emptyptr + 1];
      emptyptr++ ;
      if (emptyptr == BUFSIZE)
	      emptyptr = 0;       /* wrap araound */
      bufcnt -- ;
      if (bufcnt < 0)
      {
	      printf("underrun tmpbuf \n");
	      sem_post(&mutex);
	      exit(-1);
      }
  }
  sem_post( &mutex);

  /* Now convert  I/Q band 9000 - 12000 Hz with Fs = 48000 Hz to USB 
     start with Hilbert transform */
     filter(tempI, sI, HilbIfilt, ZifiltI, nr_samps, LP_LENGTH);
     filter(tempQ, sQ, HilbQfilt, ZifiltQ, nr_samps, LP_LENGTH);
  /* now to USB in acq_signal */
  for (i=0; i < nr_samps; i++)
  {
         acq_signal[i]= sI[i] - sQ[i];   /* sign LSB but OK due to I/Q exchange */
  }    
 
  input_samples_buffer_request = 0;
  inpbufwidx = nr_samps;

  return (nr_samps);
}

