/* 
*     File usbio2.c
*     Author M.Bos - PA0MBo
*     Date Jan 3rd 2013
*     
*     Controls LIMA SDR AVR/SI570 freq settins
*     uses usblib-dev 
*     using code samples from DG8SAQ and PE0FKO
*/



#include <stdio.h>
#include <stdlib.h>
#include <usb.h>

struct usb_dev_handle *device_handle ;

int usbCtrlMsgOut(int request, int value, int index, char *bytes, int size)
{
   return usb_control_msg(device_handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT,
                           request, value, index, bytes, size, 500);
}

int usbCtrlMsgIn(int request, int value, int index, char *bytes, int size)
{
   return usb_control_msg(device_handle,  USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
                             request, value, index, bytes, size, 500);
}

int main(int argc, char *argv[])
{
struct usb_bus *bus;
int c, i, a, result, nr_bus, nr_device;
struct usb_bus *busses;
struct usb_device *dev;
uint32_t iFreq;
uint16_t version;
double dFreq;
char freqreg[7]= {0xe3,0xc2,0xbb,0x25,0x2e,0x95,0};   /* test freq 3.545 MHz for setting regs */
uint16_t smooth;
int lima_divider= 8;
        if ( argc != 2)
        {
           printf("Usage usbio2 <freq MHz> \n");
           exit(1);
        }
        dFreq = atof( argv[1]);
        if (dFreq  <= 0.500)
        {
           printf("Freq cannot be %s should be > 0.500 MHz \n", argv[1]);
           exit(1);
        }
        setuid(0); 
        dFreq *=lima_divider ;  /* MHz x setting of jumper jp1 */
        iFreq= (uint32_t) (dFreq*(1UL <<21)); 
    	usb_init();
    	usb_find_busses();
    	usb_find_devices();
        /* usb_set_debug(255); */
    	busses = usb_get_busses();
        /* printf("after get busses \n");   */
        nr_bus = 0; 
    	for (bus = busses; bus; bus = bus->next) {
                nr_bus++ ;
                nr_device = 0;
    		for (dev = bus->devices; dev; dev = dev->next) {
                         nr_device++;
                        /*  printf("nr_bus %d  nr_device %d Class %d\n",
                            nr_bus, nr_device, dev->descriptor.bDeviceClass);
                          printf("nr_bus %d  nr_device %d idVendor  %d\n",
                            nr_bus, nr_device, dev->descriptor.idVendor);
                          printf("nr_bus %d  nr_device %d idProduct  %d\n",
                            nr_bus, nr_device, dev->descriptor.idProduct); 
                        */

    			 /* Check if this device is the LIMA SDR */

    			if ((dev->descriptor.idVendor == 5824)&&(dev->descriptor.idProduct==1500)) {
    				/* Open the device, claim the interface and do the processing */
                         /* printf("found LIMA SDR \n"); */
                         device_handle = usb_open(dev);
                         if (device_handle <0)
                         {
                           printf("error opening Si570 device\n");
                         } 
                         else
                         {
                           /* printf("succcessfully opened the Si570 device\n" ); */
                         }
                         result = usb_claim_interface(device_handle, 0 );
                         if (result >= 0)
                         {
                           /* printf("successfully claimed interface 0 \n"); */
                          }
                         else 
                         {
                                printf("error claiming interface  result is %d\n", result);
                                 exit(1);
                         } 
                         result= usbCtrlMsgIn(0x00, 0xabcd, 0, (char *)&version, sizeof(version));
                         /* printf("result is %d version is %8x\n", result, version); */
                         if (result < 0)
                         {
                             printf("comm error with sI570 \n");
                             exit(1);
                         }
                         result = usbCtrlMsgOut(0x32, 0x55 | ( 7 << 8) , 0, (char *)&iFreq, 4 );
                         /* printf("result cmd 0x32 is %d\n", result); */
                         /*  alternative is  setting freq registers directly
                             and doing the register settings calculation here */
                         /*    result= usbCtrlMsgOut(0x30, 0x55 | ( 7 << 8),  0, (char *)freqreg, 6); */
                         /* show the register settings that are in use now */
                         result = usbCtrlMsgIn(0x3F, 0x55, 0, (char *)freqreg,6);
                         if (result < 0)
                         {
                            printf("error ctrl freq si570 by register\n");
                            exit(1);
                         }
                         else
                         {
                             /*  printf("result %d Freq from 0x3F was %x %x %x %x %x %x\n",result,
                              freqreg[0], freqreg[1], freqreg[2], freqreg[3], freqreg[4], freqreg[5]);
                              result = usb_release_interface(device_handle, 0);
                              printf("result releasing interface is %d\n", result);
                              result = usb_close(device_handle);
                              printf("result closing device is %d\n", result); */
                         }
    		}
    	}
   }
  return 0;
}

